"use strict";

this.name    = "AI Trading Purchase Book";
this.version = "2.12.WIP";

/* ====================================================================================
			CONVERSION TABLE FOR COMMODITY NAMES TO LEDGER NUMBERS
======================================================================================= */

this.$commodities = ["food", "textiles", "radioactives", "slaves", "liquor_wines", "luxuries", "narcotics", "computers", "machinery", "alloys", "firearms", "furs", "minerals", "gold", "platinum", "gem_stones", "alien_items"];

/* ====================================================================================
			RECORD OF OPEN TRADES

	If "CargoTypeExtension" (New Cargoes OXP) is found, the sorting order is reversed
	so the most expensive purchases - which are likely to correspond to specialty 
	cargo types - appear last.
======================================================================================= */

    this.startUp = function () {
        "use strict";
        if (missionVariables.tradingAI_purchaseBook) {
            this.$purchaseBook = JSON.parse(missionVariables.tradingAI_purchaseBook);
        } else {
            this._setUp();
        }
        if (worldScripts.CargoTypeExtension) {
            this._sortPurchaseBook = this._sortPurchaseBookAscending;
        } else {
            this._sortPurchaseBook = this._sortPurchaseBookDescending;
        }
        if (worldScripts["AI Trading Assistant"] &&
            worldScripts["AI Trading Assistant"]._addDisplayMode) {
            this.$myModeNumber = worldScripts["AI Trading Assistant"]._addDisplayMode(["purchase book", "AI Trading Purchase Book", "_columnDisplay", "AI Trading Assistant", "_setCostHUD"]);
        }
        delete this._setUp;
        delete this._sortPurchaseBookAscending;
        delete this._sortPurchaseBookDescending;
        delete this.startUp;
    }

    this.playerWillSaveGame = function () {
        "use strict";
        missionVariables.tradingAI_purchaseBook = JSON.stringify(this.$purchaseBook);
    }

    this._setUp = function () {
        "use strict";
        this.$purchaseBook = [[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[]];
        var i, cargo, quantity, price;
        if (worldScripts['AI Trading Assistant'] &&
            worldScripts['AI Trading Assistant'].startUp) {
            worldScripts['AI Trading Assistant'].startUp();
        }
        if (worldScripts['AI Trading Assistant'] &&
            worldScripts['AI Trading Assistant'].$averagesPaid) {
            for (i = 0; i < 17; i++) {
                cargo = this.$commodities[i];
                quantity = manifest[cargo];
                price = worldScripts['AI Trading Assistant'].$averagesPaid[i] * 10; // Credits to deci-credits.
                if (quantity > 0) {
                    this.$purchaseBook[i].push([quantity, price, galaxyNumber, system.ID]);
                }
            }
        } else {	// Get from current station...
            for (i = 0; i < 17; i++) {
                cargo = this.$commodities[i];
                quantity = manifest[cargo];
                price = player.ship.dockedStation.market[cargo].price;
                if (quantity > 0) {
                    this.$purchaseBook[i].push([quantity, price, galaxyNumber, system.ID]);
                }
            }
        }
    }

/* ====================================================================================
			PLAYER BUYING CARGO
======================================================================================= */

    this.playerBoughtCargo = function (cargo, quantity, price) {
        "use strict";
        var ledgerNumber = this.$commodities.indexOf(cargo);
        if (this.$purchaseBook[ledgerNumber][0] &&
            this.$purchaseBook[ledgerNumber][0][1] == price &&
            this.$purchaseBook[ledgerNumber][0][2] == galaxyNumber &&
            this.$purchaseBook[ledgerNumber][0][3] == system.ID) {
            this.$purchaseBook[ledgerNumber][0][0] += quantity;
        } else {
            this.$purchaseBook[ledgerNumber].unshift([quantity, price, galaxyNumber, system.ID]);
        }
    }

/* ====================================================================================
			PLAYER SELLING CARGO

	Based on code originally supplied by Mika Spara.
======================================================================================= */

    this.playerSoldCargo = function (cargo, quantity, price) {
        "use strict";
        var ledgerNumber = this.$commodities.indexOf(cargo);
        var firstEntry;
        while (quantity > 0 && this.$purchaseBook[ledgerNumber].length > 0) {
            firstEntry = this.$purchaseBook[ledgerNumber].shift();
            if (firstEntry[0] > quantity) { // More than enough goods in first entry...
                firstEntry[0] -= quantity;
                // ...so remainder gets unshifted...
                this.$purchaseBook[ledgerNumber].unshift(firstEntry);
                quantity = 0;
            } else {
                quantity -= firstEntry[0];
            }
        }
    }

/* ====================================================================================
			DISPLAY COLUMN OF PURCHASE COSTS AS CONSOLE MESSAGE
======================================================================================= */

    this._purchaseBookDisplay = function () {
        "use strict";
        var message = " ";
        var thisLine, quantityPadding, quantity, pricePadding, price;
        var arrayLength = this.$purchaseBook.length;
        for (var i = 0; i < arrayLength; i++) {
            if (this.$purchaseBook[i].length == 0) {
                message = message + "\n";
            } else {
                thisLine = "         ";
                quantityPadding = "";
                quantity = this.$purchaseBook[i][0][0];
                pricePadding = "";
                price = this.$purchaseBook[i][0][1];
                if (quantity < 10) {
                    quantityPadding = "  ";
                }
                if (price < 100) {
                    pricePadding = "    ";
                } else if (price < 1000) {
                    pricePadding = "  ";
                }
                thisLine += quantityPadding + quantity + "\xD7" + (price / 10).toFixed(1) + pricePadding;
                if (this.$purchaseBook[i].length > 1) {
                    thisLine += " >";
                } else {
                    thisLine += "   ";
                }
                message = message + "\n" + thisLine;
            }
        }
        return message;
    }

    this._columnDisplay = function () {
        "use strict";
        var message = worldScripts["AI Trading Purchase Book"]._purchaseBookDisplay();
        return message;
    }

/* ====================================================================================
			SORT PURCHASE BOOK WHEN SHIP DOCKS WITH NEW STATION

	The function _sortPurchaseBook is not defined here, but is copied from either
	_sortPurchaseBookDescending or _sortPurchaseBookAscending, as required.
======================================================================================= */

    this.shipLaunchedFromStation = function (station) {
        "use strict";
        this.$lastStationVisited = station;
    }

    this.shipDockedWithStation = function (station) {
        "use strict";
        if (station !== this.$lastStationVisited) {
            this._sortPurchaseBook();
        }
    }

    this._sortPurchaseBook = function () {
        "use strict";
    }

    this._sortPurchaseBookDescending = function () {
        "use strict";
    
        function descendingSort(a, b) {
            return b[1] - a[1];
        }
    
        for (var i = 0; i < this.$purchaseBook.length; i++) {
            if (this.$purchaseBook[i].length > 1) {
                this.$purchaseBook[i].sort(descendingSort);
            }
        }
    }

    this._sortPurchaseBookAscending = function () {
        "use strict";
    
        function ascendingSort(a, b) {
            return a[1] - b[1];
        }
    
        for (var i = 0; i < this.$purchaseBook.length; i++) {
            if (this.$purchaseBook[i].length > 1) {
                this.$purchaseBook[i].sort(ascendingSort);
            }
        }
    }

/* ====================================================================================
			COPYRIGHT NOTICE

	(C)2013, Wildeblood. All rights reserved. 
	Free to use, but DO NOT PROMULGATE MODIFIED VERSIONS.

This was created by some fellow with the user name "Wildeblood" on the Oolite forums, 
he'll tell you his real name if you bother to ask.
To use means to play with, modify or re-use in whole or in part, and to distribute freely.
To use DOES NOT imply to use for commercial gain, which would be a breach of copyright.
To promulgate means to make official announcements.
======================================================================================= */
